<?php

/**
 * AsalaeCore\Command\AdminCommand
 */

namespace AsalaeCore\Command;

use AsalaeCore\Controller\Component\AclComponent;
use AsalaeCore\Exception\GenericException;
use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Controller\Controller;
use Cake\Http\ServerRequest;
use Cake\ORM\TableRegistry;
use Exception;

/**
 * Command d'install d'admin fonctionnel
 *
 * @category    Command
 *
 * @author      Libriciel SCOP <contact@libriciel.coop>
 * @copyright   (c) 2019, Libriciel
 * @license     https://www.gnu.org/licenses/agpl-3.0.txt
 */
class AdminWebserviceCommand extends AdminCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'admin webservice';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserWebservice = new ConsoleOptionParser();
        $parserWebservice->addArgument(
            'username',
            [
                'help' => __("Identifiant de connexion pour le webservice"),
                'required' => true,
            ]
        );
        $parserWebservice->addArgument(
            'password',
            [
                'help' => __("Mot de passe pour le webservice"),
                'required' => true,
            ]
        );

        return $parserWebservice;
    }

    /**
     * méthode principale
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;
        $this->webservice($args->getArgument('username'), $args->getArgument('password'));
    }

    /**
     * Ajoute un webservice administrateur
     * @param string $username
     * @param string $password
     */
    public function webservice(string $username, string $password)
    {
        $Users = TableRegistry::getTableLocator()->get('Users');
        $OrgEntities = TableRegistry::getTableLocator()->get('OrgEntities');
        $se = $OrgEntities->find()
            ->innerJoinWith('TypeEntities')
            ->where(['TypeEntities.code' => 'SE'])
            ->firstOrFail();
        if ($Users->exists(['username' => $username])) {
            throw new GenericException("$username already exists");
        }
        $user = $Users->newEntity(
            [
                'username' => $username,
                'password' => $password,
                'high_contrast' => false,
                'active' => true,
                'org_entity_id' => $se->id,
                'agent_type' => 'software',
                'is_validator' => false,
                'use_cert' => false,
            ]
        );
        $conn = $Users->getConnection();
        $conn->begin();
        $Users->saveOrFail($user);
        $request = new ServerRequest();
        $controller = new Controller($request);
        $registry = $controller->components();
        $Acl = new AclComponent($registry);
        $aro = ['model' => 'Users', 'foreign_key' => $user->id];
        $Acl->allow($aro, 'root/api');
        $Acl->deny($aro, 'root/controllers');
        $conn->commit();
        $this->io->out('new user id=' . $user->id);
    }
}
