<?php

namespace AsalaeCore\Command;

use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Console\ConsoleOptionParser as CakeConsoleOptionParser;
use Cake\Core\Configure;
use Cake\Error\Debugger;
use Cake\Utility\Inflector;
use Exception;

class ControllersGetCommand extends ControllersSetCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'controllers get';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return CakeConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): CakeConsoleOptionParser
    {
        $parserGet = new ConsoleOptionParser();
        $parserGet->addArgument(
            'target',
            [
                'help' => __("Action sous la forme Controller/action"),
            ]
        );
        $parserGet->addOption(
            'plugin',
            [
                'help' => __("Cible un plugin"),
            ]
        );
        return $parserGet;
    }

    /**
     * Donne le paramétrage effectué pour une action
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;
        $target = $this->args->getArgument('target');
        while (
            empty($target)
            || !preg_match('/^([A-Za-z\d.-]+)\/([A-Za-z\d-]+)$/', $target, $match)
            || !$this->checkExistance($match[1], $match[2])
        ) {
            $this->io->out("La cible doit être indiquée sous la forme Controllers/actionEnCamelCase");
            $target = $this->io->ask(__("Veuillez saisir une cible"));
        }
        $pathToControllersJson = Configure::read(
            'App.paths.controllers_rules',
            RESOURCES . 'controllers.json'
        );
        $data = json_decode(file_get_contents($pathToControllersJson), true);
        [, $controller, $action] = $match;
        $controller = Inflector::camelize($controller, '-');
        $action = Inflector::variable(Inflector::camelize($action, '-'));
        if (!isset($data[$controller])) {
            $data[$controller] = [];
        }
        if (!isset($data[$controller][$action])) {
            $data[$controller][$action] = [];
        }
        $this->io->out(Debugger::exportVarAsPlainText($data[$controller][$action]));
    }
}
