<?php

namespace AsalaeCore\Command;

use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Core\Configure;
use Cake\Utility\Hash;
use Exception;

class ControllersListCommand extends ControllersSetCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'controllers list';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserList = new ConsoleOptionParser();
        $parserList->addArgument(
            'controller',
            [
                'help' => __("Filtre les résultats sur un unique Controller"),
            ]
        );
        $parserList->addOption(
            'plugin',
            [
                'help' => __("Cible un plugin"),
            ]
        );
        return $parserList;
    }

    /**
     * méthode principale
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;

        $controllerName = $this->args->getArgument('controller');
        $controllers = $this->getControllersActions();
        $pathToControllersJson = Configure::read(
            'App.paths.controllers_rules',
            RESOURCES . 'controllers.json'
        );
        $data = json_decode(file_get_contents($pathToControllersJson), true);
        if ($controllerName) {
            $controllers = [$controllerName => $controllers[$controllerName] ?? []];
        }
        $error = 0;
        foreach ($controllers as $controller => $actions) {
            foreach ($actions as $action) {
                if (Hash::get($data, $controller . '.' . $action)) {
                    $this->io->out("$controller/$action");
                } else {
                    $this->io->err("$controller/$action");
                    $error++;
                }
            }
        }
        if ($error) {
            $this->io->abort(__n("Il manque une action", "Il manque {0} actions", $error, $error));
        }
    }
}
