<?php

namespace AsalaeCore\Command;

use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\I18n\DateTime as I18nDateTime;
use Cake\ORM\TableRegistry;
use DateTimeInterface;
use Exception;

class CronListCommand extends CronCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'cron list';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserList = new ConsoleOptionParser('list');

        $parserList->addOption(
            'id',
            ['help' => __("Liste par ID"), 'boolean' => true]
        );

        return $parserList;
    }

    /**
     * méthode principale
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;

        $fields = ['id', 'classname', 'active', 'locked', 'frequency', 'next', 'parallelisable'];
        $data = TableRegistry::getTableLocator()->get('Crons')->find()
            ->select($fields)
            ->orderBy([$this->args->getOption('id') ? 'id' : 'classname'])
            ->disableHydration()
            ->all()
            ->map(
                function ($data) {
                    foreach ($data as $k => $v) {
                        if ($v instanceof DateTimeInterface) {
                            $data[$k] = $v->format('Y-m-d H:i:s');
                        }
                        $data[$k] = (string)$data[$k];
                    }
                    return $data;
                }
            )
            ->toArray();

        $now = new I18nDateTime();
        $today = new I18nDateTime($now->format('Y-m-d'));
        // On récupère le nom de classe sans namespace pour pouvoir le trier
        $nData = [];
        foreach ($data as $row) {
            $row['classname'] = substr(
                $row['classname'],
                strrpos($row['classname'], '\\') + 1
            );
            $row['active'] = $row['active'] ? 'true' : '';
            $row['locked'] = $row['locked'] ? 'true' : '';
            $row['parallelisable'] = $row['parallelisable'] ? 'true' : '';
            $row['next'] = new I18nDateTime($row['next']);
            $days = (new I18nDateTime($row['next']->format('Y-m-d')))->diffInDays($today);
            if ($row['next'] < $now) {
                $row['next'] = __("prochainement");
            } elseif ($days === 0) {
                debug([$row['classname'] => $row['next']]);
                $row['next'] = __("aujourd'hui à {0}", $row['next']->format('H:i:s'));
            } elseif ($days === 1) {
                $row['next'] = __("demain à {0}", $row['next']->format('H:i:s'));
            } else {
                $row['next'] = __(
                    "dans {0} jours à {1}",
                    $days,
                    $row['next']->format('H:i:s')
                );
            }
            $nData[$row['classname']] = $row;
        }
        ksort($nData);
        $nData = array_values($nData);
        array_unshift($nData, $fields);

        $this->io->helper('Table')->output($nData);
    }
}
