<?php

namespace AsalaeCore\Command;

use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Exception;

class CronRunCommand extends CronCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'cron run';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserRun = new ConsoleOptionParser();
        $parserRun->addArgument(
            'cron',
            ['help' => __("ID du cron (DB)"), 'required' => true]
        );

        $parserRun->addOption(
            'params',
            [
                'short' => 'p',
                'help' => __("Permet de passer des paramètres sous la forme key=value"),
                'multiple' => true,
            ]
        );
        $parserRun->addOption(
            'context',
            [
                'help' => __("Permet de spécifier un contexte d'execution"),
                'default' => 'shell',
            ]
        );
        $parserRun->addOption(
            'no-interaction',
            [
                'help' => __("Désactive la partie interactive"),
                'boolean' => true,
            ]
        );
        return $parserRun;
    }

    /**
     * Lance un cron
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;

        $this->runCron($this->args->getArgument('cron'));
    }
}
