<?php

namespace AsalaeCore\Command;

use AsalaeCore\Model\Table\CronExecutionsTable;
use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Datasource\EntityInterface;
use Cake\ORM\Query;
use Cake\Utility\Hash;
use Exception;

class CronUnlockCommand extends CronCommand
{
    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'cron unlock';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserUnlock = new ConsoleOptionParser('unlock');
        $parserUnlock->addArgument(
            'cron',
            ['help' => __("ID du cron (DB)"), 'required' => true]
        );
        return $parserUnlock;
    }

    /**
     * méthode principale
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;

        $Crons = $this->fetchTable('Crons');
        $CronExecutions = $this->fetchTable('CronExecutions');
        $cron = $Crons->find()
            ->where(['Crons.id' => $args->getArgument('cron')])
            ->contain(
                [
                    'CronExecutions' => function (Query $q) {
                        return $q->where(['CronExecutions.state' => CronExecutionsTable::S_RUNNING]);
                    },
                ]
            )
            ->firstOrFail();
        /** @var EntityInterface $exec */
        foreach (Hash::get($cron, 'cron_executions', []) as $exec) {
            if (file_exists('/proc/' . $exec->get('pid'))) {
                $this->io->abort(__("Le cron semble toujours actif dans le process id n°{0}", $exec->get('pid')));
            }
            $CronExecutions->updateAll(
                [
                    'state' => CronExecutionsTable::S_CANCELED,
                    'report' => trim($exec->get('report') . ' - ' . __("Annulé par un administrateur"), ' -'),
                ],
                ['id' => $exec->id]
            );
            $this->io->out(__("Execution n°{0} annulé par administrateur", $exec->id));
        }
        if ($cron->get('state') === CronExecutionsTable::S_RUNNING) {
            $cron->set('state', CronExecutionsTable::S_CANCELED);
        }
    }
}
