<?php

/**
 * AsalaeCore\Command\TreeCommand
 */

namespace AsalaeCore\Command;

use Cake\Command\Command;
use Cake\Console\Arguments;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\Core\Configure;
use Cake\I18n\DateTime;
use Cake\ORM\Behavior\TreeBehavior;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use Cake\Utility\Inflector;
use Exception;

/**
 * Permet de réparer un arbre (bdd)
 *
 * @category    Command
 *
 * @author      Libriciel SCOP <contact@libriciel.coop>
 * @copyright   (c) 2019, Libriciel
 * @license     https://www.gnu.org/licenses/agpl-3.0.txt
 */
class TreeCommand extends Command
{
    /**
     * @var Arguments
     */
    public $args;
    /**
     * @var ConsoleIo
     */
    public $io;

    /**
     * Get the command name.
     * @return string
     */
    public static function defaultName(): string
    {
        return 'tree repair';
    }

    /**
     * Gets the option parser instance and configures it.
     *
     * By overriding this method you can configure the ConsoleOptionParser before returning it.
     *
     * @return ConsoleOptionParser
     * @link https://book.cakephp.org/3.0/en/console-and-shells.html#configuring-options-and-generating-help
     */
    public function getOptionParser(): ConsoleOptionParser
    {
        $parserRepair = new ConsoleOptionParser();
        $parserRepair->addArgument(
            'model_name',
            [
                'help' => __("Nom du model sur lequel effectuer un recover ou ''all'' pour les faire tous "),
                'required' => true,
            ]
        );
        return $parserRepair;
    }

    /**
     * méthode principale
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return void
     * @throws Exception
     */
    public function execute(Arguments $args, ConsoleIo $io)
    {
        $this->args = $args;
        $this->io = $io;

        $model = $this->args->getArgument('model_name');
        if ($model === 'all') {
            $this->repairAll();
        } else {
            $this->repair($model);
        }
    }

    /**
     * Fonction de réparation
     * @param string $model
     * @throws Exception
     */
    public function repair(string $model)
    {
        $modelName = Inflector::camelize($model);
        /** @var Table|TreeBehavior $Model */
        $Model = TableRegistry::getTableLocator()->get($modelName);
        if (!$Model->hasBehavior('Tree')) {
            throw new Exception('TreeBehavior nor found on ' . $modelName);
        }
        $this->io->out(__("Réparation de {0} commencée à {1}", $modelName, (new DateTime())->toTimeString()));
        $Model->recover();
        $this->io->out(__("Réparation de {0} terminée à {1}", $modelName, (new DateTime())->toTimeString()));
    }

    /**
     * Répare tous les models compatibles
     * @throws Exception
     */
    public function repairAll()
    {
        $loc = TableRegistry::getTableLocator();
        $basePath = Configure::read('Tree.model_path', APP . 'Model' . DS . 'Table');
        foreach (glob($basePath . DS . '*Table.php') as $filename) {
            $modelname = basename($filename, 'Table.php');
            $Model = $loc->get($modelname);
            if ($Model->hasBehavior('Tree')) {
                $this->repair($modelname);
            }
        }
    }
}
