<?php

/**
 * AsalaeCore\Controller\RenderDataTrait
 */

namespace AsalaeCore\Controller;

use AsalaeCore\Http\AppServerRequest;
use AsalaeCore\Http\Response;
use Cake\Controller\Controller;
use Cake\Http\Response as CakeResponse;

/**
 * Permet le rendu d'une variable en divers formats (ex: json, xml)
 *
 * @category Controller
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 * @mixin Controller
 * @property AppServerRequest $request
 * @property Response         $response
 */
trait RenderDataTrait
{
    /**
     * Effectue un rendu de fichier csv
     * @param Controller        $controller
     * @param array             $data
     * @param CakeResponse|null $response
     * @return Response
     */
    public static function staticRenderCsv(
        Controller $controller,
        $data,
        CakeResponse $response = null
    ): CakeResponse {
        return AbstractRenderData::staticRenderCsv($controller, $data, $response);
    }

    /**
     * Effectue un json_encode sur le data avant de lancer le rendu json
     * @param mixed             $data
     * @param CakeResponse|null $response
     * @return Response
     */
    protected function renderDataToJson($data, CakeResponse $response = null): CakeResponse
    {
        return $this->renderJson(json_encode($data), $response);
    }

    /**
     * Effectue un rendu de fichier json
     * @param string            $json
     * @param CakeResponse|null $response
     * @return Response
     */
    protected function renderJson(string $json, CakeResponse $response = null): CakeResponse
    {
        return self::staticRenderJson($this, $json, $response);
    }

    /**
     * Effectue un rendu de fichier json
     * @param Controller        $controller
     * @param string            $json
     * @param CakeResponse|null $response
     * @return Response
     */
    public static function staticRenderJson(
        Controller $controller,
        string $json,
        CakeResponse $response = null
    ): CakeResponse {
        return AbstractRenderData::staticRenderJson($controller, $json, $response);
    }

    /**
     * Effectue un rendu de fichier xml
     * @param string            $xml
     * @param CakeResponse|null $response
     * @return Response
     */
    protected function renderXml(string $xml, CakeResponse $response = null): CakeResponse
    {
        return self::staticRenderXml($this, $xml, $response);
    }

    /**
     * Effectue un rendu de fichier xml
     * @param Controller        $controller
     * @param string            $xml
     * @param CakeResponse|null $response
     * @return Response
     */
    public static function staticRenderXml(
        Controller $controller,
        string $xml,
        CakeResponse $response = null
    ): CakeResponse {
        return AbstractRenderData::staticRenderXml($controller, $xml, $response);
    }

    /**
     * Effectue un rendu de fichier html
     * @param mixed             $data
     * @param CakeResponse|null $response
     * @return CakeResponse
     */
    protected function renderHtml($data, CakeResponse $response = null)
    {
        return self::staticRenderHtml($this, $data, $response);
    }

    /**
     * Effectue un rendu de fichier html
     * @param Controller        $controller
     * @param mixed             $data
     * @param CakeResponse|null $response
     * @return CakeResponse
     */
    public static function staticRenderHtml(Controller $controller, $data, CakeResponse $response = null)
    {
        return AbstractRenderData::staticRenderHtml($controller, $data, $response);
    }

    /**
     * Effectue le rendu des données selon le(s) accept(s) de la requête
     * @param array             $accepts
     * @param mixed             $output
     * @param CakeResponse|null $response
     * @return Response
     */
    protected function renderByAccepts(array $accepts, $output, CakeResponse $response = null): CakeResponse
    {
        return self::staticRenderByAccepts($this, $accepts, $output, $response);
    }

    /**
     * Effectue le rendu des données selon le(s) accept(s) de la requête
     * @param Controller        $controller
     * @param array             $accepts
     * @param mixed             $output
     * @param CakeResponse|null $response
     * @return Response
     */
    public static function staticRenderByAccepts(
        Controller $controller,
        array $accepts,
        $output,
        CakeResponse $response = null
    ): CakeResponse {
        return AbstractRenderData::staticRenderByAccepts($controller, $accepts, $output, $response);
    }

    /**
     * Effectue un rendu selon le accept
     * @param mixed             $output
     * @param CakeResponse|null $response
     * @return Response
     */
    protected function renderData($output, CakeResponse $response = null): CakeResponse
    {
        return self::staticRenderData($this, $output, $response);
    }

    /**
     * Effectue un rendu selon le accept
     * @param Controller        $controller
     * @param mixed             $output
     * @param CakeResponse|null $response
     * @return Response
     */
    public static function staticRenderData(
        Controller $controller,
        $output,
        CakeResponse $response = null
    ): CakeResponse {
        return AbstractRenderData::staticRenderData($controller, $output, $response);
    }
}
