<?php

/**
 * AsalaeCore\Database\Type\DateTimeType
 */

namespace AsalaeCore\Database\Type;

use Cake\Database\Type\BatchCastingInterface;
use Cake\Database\Type\DateTimeType as CakeDateTimeType;
use Cake\Database\TypeInterface;
use Cake\I18n\DateTime;
use DateTimeImmutable;

/**
 * Permet la triple compatibilité de dates : format DATE_RFC3339, format local et objet
 *
 * @category Database
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2020, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 */
class DateTimeType extends CakeDateTimeType implements TypeInterface, BatchCastingInterface
{
    /**
     * Parse la date
     * @param string $value
     * @return DateTime|null
     */
    protected function _parseLocaleValue(string $value): ?DateTime
    {
        if (!$value) {
            return null;
        }
        $date = $this->_parseValue($value);
        return $date instanceof DateTime ? $date : parent::_parseLocaleValue($value);
    }

    /**
     * Converts a string into a DateTime object after parsing it using the locale
     * aware parser with the specified format.
     *
     * @param string $value The value to parse and convert to an object.
     * @return DateTimeImmutable|DateTime|null
     */
    protected function _parseValue(string $value): DateTime|DateTimeImmutable|null
    {
        /** @var DateTime|DateTimeImmutable $class */
        $class = $this->_className;
        // Si format DATE_RFC3339
        if (preg_match('/^(\d{4}-\d{2}-\d{2}([ T]\d{2}:\d{2}(:\d{2})?)?|\d{2}:\d{2}(:\d{2})?)/', $value)) {
            return new $class($value);
        }

        return $class::parseDateTime($value, $this->_localeMarshalFormat);
    }
}
