<?php

/**
 * AsalaeCore\Error\Renderer\JsonExceptionRenderer
 */

namespace AsalaeCore\Error\Renderer;

use AsalaeCore\Controller\AbstractRenderData;
use Cake\Error\ExceptionRendererInterface;
use Cake\Error\Renderer\WebExceptionRenderer;
use Cake\Http\Exception\HttpException;
use Psr\Http\Message\ResponseInterface;

use function Cake\Core\h;

/**
 * Affichage des exceptions en json
 *
 * @category    Error
 *
 * @author      Libriciel SCOP <contact@libriciel.coop>
 * @copyright   (c) 2024, Libriciel
 * @license     https://www.gnu.org/licenses/agpl-3.0.txt
 */
class JsonExceptionRenderer extends WebExceptionRenderer implements ExceptionRendererInterface
{
    /**
     * Renders the response for the exception.
     *
     * @return ResponseInterface The response to be sent.
     */
    public function render(): ResponseInterface
    {
        $exception = $this->error;
        $code = $this->getHttpCode($exception);
        $this->clearOutput();

        $message = $this->_message($exception, $code);
        $url = $this->controller->getRequest()->getRequestTarget();
        $response = $this->controller->getResponse();

        if ($exception instanceof HttpException) {
            foreach ($exception->getHeaders() as $name => $value) {
                $response = $response->withHeader($name, $value);
            }
        }
        $response = $response->withStatus($code);

        $viewVars = [
            'message' => $message,
            'url' => h($url),
            'error' => (string)$exception,
            'code' => $code,
        ];
        return AbstractRenderData::staticRenderJson($this->_getController(), json_encode($viewVars), $response);
    }
}
