<?php

namespace AsalaeCore\Form\MessageSchema;

/**
 * Partie DataObjectPackage d'un message seda v2.1
 *
 * @category Form\MessageSchema
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2024, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 */
trait Seda21DataObjectPackageTrait
{
    /**
     * ArchiveTransfer.DataObjectPackage
     * @return array
     */
    protected function getDataObjectPackage(): array
    {
        return [
            'text' => 'DataObjectPackage',
            'cardinality' => '0..1',
            'li_attr' => [
                'title' => __("Objets-données échangés dans le message."),
            ],
            'children' => $this->list(
                'DataObjectGroup',
                'BinaryDataObject',
                'PhysicalDataObject',
                'DescriptiveMetadata',
                'ManagementMetadata'
            ),
            'complex' => [
                [
                    'choice' => [
                        'DataObjectGroup',
                        'BinaryDataObject',
                        'PhysicalDataObject',
                    ],
                ],
                'DescriptiveMetadata',
                'ManagementMetadata',
            ],
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject
     * @return array
     */
    protected function getBinaryDataObject(): array
    {
        return [
            'text' => 'BinaryDataObject',
            'cardinality' => '0..n',
            'true_cardinality' => '1..1',
            'li_attr' => [
                'title' => __("Objet-données numérique."),
            ],
            'attr' => [
                'id' => [
                    'required' => true,
                    'label' => __("Identifiant de l'objet-données associé."),
                ],
            ],
            'children' => $this->list(
                $this->extMinimalDataObjectType(),
                'Attachment',
                'Uri',
                'MessageDigest',
                'Size',
                'Compressed',
                'FormatIdentification',
                'FileInfo',
                'Metadata',
                'OtherMetadata',
                [
                    'custom' => [
                        'type' => 'hidden',
                        'default' => json_encode(
                            [
                                'action' => 'multiAction',
                                'args' => [
                                    [
                                        'action' => 'unsetValue',
                                        'args' => ['Attachment'],
                                    ],
                                    [
                                        'action' => 'addBinaryDataObject',
                                    ],
                                ],
                            ]
                        ),
                    ],
                ]
            ),
            'complex' => [
                'DataObjectSystemId',
                'DataObjectGroupSystemId',
                'Relationship',
                'DataObjectGroupReferenceId',
                'DataObjectGroupId',
                'DataObjectVersion',
                [
                    'choice' => [
                        'Attachment',
                        'Uri',
                    ],
                ],
                'MessageDigest',
                'Size',
                'Compressed',
                'FormatIdentification',
                'FileInfo',
                'Metadata',
                'OtherMetadata',
            ],
            'choice' => [
                'Attachment',
                'Uri',
            ],
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.DataObjectVersion
     * @return array
     */
    protected function getDataObjectVersion(): array
    {
        return [
            'label' => "DataObjectVersion",
            'info' => __(
                "Version d’un objet-données (par exemple : original papier, conservation, diffusion, vignette, txt, …)."
            ),
            'cardinality' => '0..1',
            'attributes' => $this->attrs('IdentifierType'),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.Attachment
     * @return array
     */
    protected function getAttachment(): array
    {
        return [
            'label' => "Attachment : " . __("Objet-données"),
            'cardinality' => '0..1',
            'true_cardinality' => '1..1',
            'attributes' => $this->attrs('BinaryObjectType'),
            'class' => 'attachments-files-select',
            'onchange' => 'loadAttachment(this)',
            'afterload' => 'initializeAttachment',
            'chosen' => false,
            'options' => [],
            'data-placeholder' => __("-- Sélectionner un fichier --"),
            'allowEmpty' => true, // force la validation à se désactiver car la valeur est set uniquement pour le js
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.MessageDigest
     * @return array
     */
    protected function getMessageDigest(): array
    {
        return [
            'label' => "MessageDigest",
            'cardinality' => '0..1',
            'true_cardinality' => '1..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.Size
     * @return array
     */
    protected function getSize(): array
    {
        return [
            'label' => "Size",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FormatIdentification
     * @return array
     */
    protected function getFormatIdentification(): array
    {
        return [
            'text' => 'FormatIdentification',
            'cardinality' => '0..1',
            'li_attr' => [
                'title' => __("Identification du format de l'objet-données."),
            ],
            'disabled' => true,
            'children' => $this->list(
                'FormatLitteral',
                'MimeType',
                'FormatId',
                'Encoding'
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FormatLitteral
     * @return array
     */
    protected function getFormatLitteral(): array
    {
        return [
            'label' => "FormatLitteral",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.MimeType
     * @return array
     */
    protected function getMimeType(): array
    {
        return [
            'label' => "MimeType",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FormatId
     * @return array
     */
    protected function getFormatId(): array
    {
        return [
            'label' => "FormatId",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo
     * @return array
     */
    protected function getFileInfo(): array
    {
        return [
            'text' => 'FileInfo',
            'cardinality' => '0..1',
            'li_attr' => [
                'title' => __("Informations sur le fichier lui-même (d'un point de vue technique)."),
            ],
            'disabled' => true,
            'children' => $this->list(
                'Filename',
                'CreatingApplicationName',
                'CreatingApplicationVersion',
                'DateCreatedByApplication',
                'CreatingOs',
                'CreatingOsVersion',
                'LastModified'
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.Filename
     * @return array
     */
    protected function getFilename(): array
    {
        return [
            'label' => "Filename",
            'cardinality' => '1..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.CreatingApplicationName
     * @return array
     */
    protected function getCreatingApplicationName(): array
    {
        return [
            'label' => "CreatingApplicationName",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.CreatingApplicationVersion
     * @return array
     */
    protected function getCreatingApplicationVersion(): array
    {
        return [
            'label' => "CreatingApplicationVersion",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.DateCreatedByApplication
     * @return array
     */
    protected function getDateCreatedByApplication(): array
    {
        return [
            'label' => "DateCreatedByApplication",
            'cardinality' => '0..1',
            'disabled' => true,
            'data-type' => 'datetime',
            'validations' => $this->validations('DateTimeType'),
            'class' => 'datepicker',
            'placeholder' => $this->datePlaceholder(),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.CreatingOs
     * @return array
     */
    protected function getCreatingOs(): array
    {
        return [
            'label' => "CreatingOs",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.LastModified
     * @return array
     */
    protected function getLastModified(): array
    {
        return [
            'label' => "LastModified",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.BinaryDataObject.FileInfo.CreatingOsVersion
     * @return array
     */
    protected function getCreatingOsVersion(): array
    {
        return [
            'label' => "CreatingOsVersion",
            'cardinality' => '0..1',
            'disabled' => true,
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata
     * @return array
     */
    protected function getDescriptiveMetadata(): array
    {
        return [
            'text' => 'DescriptiveMetadata',
            'cardinality' => '1..1',
            'li_attr' => [
                'title' => __(
                    "Bloc de métadonnées descriptives des objets-données."
                ),
            ],
            'children' => $this->list(
                'ArchiveUnit'
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit
     * @return array
     */
    protected function getArchiveUnit(): array
    {
        return [
            'text' => 'ArchiveUnit',
            'cardinality' => '0..n',
            'true_cardinality' => '1..1',
            'li_attr' => [
                'title' => __(
                    "Correspond à la notion de composant en ISAD(G). ArchiveUnit "
                    . "permet à la fois de gérer la hiérarchie intellectuelle, tout en contenant "
                    . "les métadonnées de description et de gestion propres à chaque niveau de "
                    . "description archivistique."
                ),
            ],
            'attr' => [
                'id' => [
                    'required' => true,
                    'label' => __("Identifiant de l'objet-données associé."),
                ],
            ],
            'children' => $this->list(
                'ArchiveUnitRefId',
                'ArchiveUnitProfile',
                'Management',
                'Content',
                'ArchiveUnit',
                'DataObjectReference',
                [
                    'custom' => [
                        'type' => 'hidden',
                        'default' => json_encode(
                            [
                                'action' => 'addUUID',
                                'args' => [],
                            ]
                        ),
                    ],
                ]
            ),
            'complex' => [
                [
                    'choice' => [
                        'ArchiveUnitRefId',
                        [
                            'sequence' => [
                                'ArchiveUnitProfile',
                                'Management',
                                'Content',
                                'ArchiveUnit',
                                'DataObjectReference',
                            ],
                        ],
                    ],
                ],
            ],
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit.ArchiveUnitRefId
     * @return array
     */
    protected function getArchiveUnitRefId(): array
    {
        return [
            'label' => 'ArchiveUnitRefId',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __("Permet de faire une référence à d'autres ArchiveUnit dans la même transaction."),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit.ArchiveUnitProfile
     * @return array
     */
    protected function getArchiveUnitProfile(): array
    {
        return [
            'label' => 'ArchiveUnitProfile',
            'cardinality' => '0..1',
            'info' => __(
                "Référence à une partie d'un profil d’archivage applicable à un"
                . " ArchiveUnit en particulier. Permet par exemple de faire référence à "
                . "une typologie documentaire dans un profil d'archivage."
            ),
            'attributes' => $this->attrs('IdentifierType'),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit.Management
     * @return array
     */
    protected function getManagement(): array
    {
        return [
            'text' => 'Management',
            'cardinality' => '0..1',
            'li_attr' => [
                'title' => __("Métadonnées de gestion applicables à l’ArchiveUnit concernée et à ses héritiers."),
            ],
            'children' => $this->list(
                $this->refManagementType()
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DataObjectGroup
     * @return array
     */
    protected function getDataObjectGroup(): array
    {
        return [
            'text' => 'DataObjectGroup',
            'cardinality' => '0..n',
            'true_cardinality' => '1..1',
            'children' => $this->list(
                'BinaryDataObject',
                'PhysicalDataObject',
                'LogBook',
                [
                    'custom' => [
                        'type' => 'hidden',
                        'default' => json_encode(
                            [
                                'action' => 'addUUID',
                                'args' => [],
                            ]
                        ),
                    ],
                ]
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit.Content
     * @return array
     */
    protected function getContent(): array
    {
        return [
            'text' => 'Content',
            'cardinality' => '1..1',
            'li_attr' => [
                'title' => __("Métadonnées de description associées à un ArchiveUnit."),
            ],
            'children' => $this->list(
                'DescriptionLevel',
                'Title',
                'FilePlanPosition',
                'SystemId',
                'OriginatingSystemId',
                'ArchivalAgencyArchiveUnitIdentifier',
                'OriginatingAgencyArchiveUnitIdentifier',
                'TransferringAgencyArchiveUnitIdentifier',
                'Description',
                'CustodialHistory',
                'Type',
                'DocumentType',
                'Language',
                'DescriptionLanguage',
                'Status',
                'Version',
                'Tag',
                'Keyword',
                'Coverage',
                'OriginatingAgency',
                'SubmissionAgency',
                'AuthorizedAgent',
                'Writer',
                'Addressee',
                'Recipient',
                'Transmitter',
                'Sender',
                'Source',
                'RelatedObjectReference',
                'CreatedDate',
                'TransactedDate',
                'AcquiredDate',
                'SentDate',
                'ReceivedDate',
                'RegisteredDate',
                [
                    'StartDate' => [
                        'label' => "StartDate : " . __("Date d'ouverture / date de début."),
                        'cardinality' => '0..1',
                        'help' => __("Optionnel"),
                        'data-type' => 'datetime',
                        'validations' => $this->validations('DateTimeType'),
                        'class' => 'datepicker',
                        'placeholder' => $this->datetimePlaceholder(),
                    ],
                ],
                'EndDate',
                'Event',
                'Signature',
                'Gps'
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata.ArchiveUnit.DataObjectReference
     * @return array
     */
    protected function getDataObjectReference(): array
    {
        return [
            'text' => 'DataObjectReference',
            'cardinality' => '0..n',
            'true_cardinality' => '1..1',
            'li_attr' => [
                'title' => __(
                    "Référence à un objet-données ou à un groupe d'objets-données interne(s)."
                ),
            ],
            'children' => $this->list(
                $this->refDataObjectRefType()
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata
     * .ArchiveUnit.Content.RelatedObjectReference.RepositoryArchiveUnitPID
     * @return array
     */
    protected function getRepositoryArchiveUnitPID(): array
    {
        return [
            'label' => 'RepositoryArchiveUnitPID',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Référence à un ArchiveUnit déjà conservé dans un système d'archivage."
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata
     * .ArchiveUnit.Content.RelatedObjectReference.RepositoryObjectPID
     * @return array
     */
    protected function getRepositoryObjectPID(): array
    {
        return [
            'label' => 'RepositoryObjectPID',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Référence à un un objet-données ou à un groupe d'objets-données"
                . " déjà conservé(s) dans un système d'archivage."
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.DescriptiveMetadata
     * .ArchiveUnit.Content.RelatedObjectReference.ExternalReference
     * @return array
     */
    protected function getExternalReference(): array
    {
        return [
            'label' => 'ExternalReference : ' . __("Référence externe."),
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata
     * @return array
     */
    protected function getManagementMetadata(): array
    {
        return [
            'text' => 'ManagementMetadata',
            'cardinality' => '0..1',
            'li_attr' => [
                'title' => __(
                    "Bloc des métadonnées de gestion par défaut des objets-données."
                ),
            ],
            'children' => $this->list(
                'ArchivalProfile',
                'ServiceLevel',
                'AcquisitionInformation',
                'LegalStatus',
                'OriginatingAgencyIdentifier',
                'SubmissionAgencyIdentifier',
                $this->refManagementType()
            ),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.ArchivalProfile
     * @return array
     */
    protected function getArchivalProfile(): array
    {
        return [
            'label' => 'ArchivalProfile',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Profil d’archivage applicable aux ArchiveUnit."
            ),
            'attributes' => $this->attrs('IdentifierType'),
            'options' => $this->optionsArchivalProfile(),
            'empty' => true,
            'data-placeholder' => __("-- Choisir un profil d'archives --"),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.ServiceLevel
     * @return array
     */
    protected function getServiceLevel(): array
    {
        return [
            'label' => 'ServiceLevel',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Niveau de service applicable aux unités d’archives."
            ),
            'attributes' => $this->attrs('IdentifierType'),
            'options' => $this->optionsServiceLevels(),
            'empty' => true,
            'data-placeholder' => __("-- Choisir un niveau de service --"),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.AcquisitionInformation
     * @return array
     */
    protected function getAcquisitionInformation(): array
    {
        return [
            'label' => 'AcquisitionInformation : ' . __("Modalités d'entrée des archives."),
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.LegalStatus
     * @return array
     */
    protected function getLegalStatus(): array
    {
        return [
            'label' => 'LegalStatus : ' . __("Statut des archives échangées."),
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'options' => [
                'Public Archive' => 'Public Archive',
                'Private Archive' => 'Private Archive',
                'Public and Private Archive' => 'Public and Private Archive',
            ],
            'empty' => __("-- Veuillez sélectionner un statut --"),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.OriginatingAgencyIdentifier
     * @return array
     */
    protected function getOriginatingAgencyIdentifier(): array
    {
        return [
            'label' => 'OriginatingAgencyIdentifier',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Identifiant du service producteur - information de "
                . "gestion à ne pas confondre avec OriginatingAgency dans les "
                . "métadonnées de description."
            ),
            'attributes' => $this->attrs('IdentifierType'),
        ];
    }

    /**
     * ArchiveTransfer.DataObjectPackage.ManagementMetadata.SubmissionAgencyIdentifier
     * @return array
     */
    protected function getSubmissionAgencyIdentifier(): array
    {
        return [
            'label' => 'SubmissionAgencyIdentifier',
            'cardinality' => '0..1',
            'help' => __("Optionnel"),
            'info' => __(
                "Identifiant du service versant - information de gestion à ne"
                . "pas confondre avec SubmissionAgency dans les métadonnées de"
                . "description."
            ),
            'attributes' => $this->attrs('IdentifierType'),
        ];
    }
}
