<?php

/**
 * AsalaeCore\MinkSuite\HttpClient
 */

namespace AsalaeCore\MinkSuite;

use DMore\ChromeDriver\HttpClient as DMoreHttpClient;
use RuntimeException;

/**
 * Surcharge du client HTTP de mink
 *
 * @category TestSuite
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2024, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 */
class HttpClient extends DMoreHttpClient
{
    public array $additionnalCurl = [
        CURLOPT_TIMEOUT => 10,
        CURLOPT_CONNECTTIMEOUT => 5,
    ];

    /**
     * Surcharge de request pour ajouter des options curl
     * @param string $url
     * @param string $verb
     * @return bool|string
     */
    protected function request($url, $verb)
    {
        $curl = curl_init();
        if ($curl === false) {
            throw new RuntimeException('Error while initializing curl');
        }
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

        // Partie surchargée
        if (!empty($this->additionnalCurl)) {
            curl_setopt_array($curl, $this->additionnalCurl);
        }
        // fin de partie surchargée

        if ('GET' !== $verb) {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $verb);
        }
        // To prevent DNS rebinding attacks, Chrome 66 and later only accepts
        // Host header with IP address or localhost.
        $parsed_url = parse_url($url);
        if ($parsed_url['host'] != 'localhost') {
            $host = gethostbyname($parsed_url['host']);
            if ($host != $parsed_url['host']) {
                if (!empty($parsed_url['port'])) {
                    $host .= ':' . $parsed_url['port'];
                }
                curl_setopt($curl, CURLOPT_HTTPHEADER, ["Host: $host"]);
            }
        }
        return curl_exec($curl);
    }
}
