<?php

namespace AsalaeCore\Utility;

use Cake\Database\Connection;
use Cake\Database\Driver;
use Cake\Datasource\ConnectionManager;
use PDO;
use ReflectionClass;
use ReflectionException;

/**
 * Raccourcis pour manipuler les méthodes PDO qui ne sont plus accéssible en
 * Cakephp 5
 *
 * @category Utility
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2019, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 */
class DatabaseUtility
{
    /**
     * @param PDO    $pdo
     * @param string $datasource
     * @return void
     */
    public static function setPdo(PDO $pdo, string $datasource = 'default'): void
    {
        $driver = self::getDriver($datasource);
        $reflection = new ReflectionClass($driver);
        $property = $reflection->getProperty('pdo');
        $property->setValue($driver, $pdo);
    }

    /**
     * @param string $datasource
     * @return Driver
     */
    public static function getDriver(string $datasource = 'default'): Driver
    {
        /** @var Connection $connection */
        $connection = ConnectionManager::get($datasource);
        return $connection->getDriver();
    }

    /**
     * @param string $datasource
     * @return string
     * @throws ReflectionException
     */
    public static function driverName(string $datasource = 'default'): string
    {
        return self::getPdo($datasource)->getAttribute(PDO::ATTR_DRIVER_NAME);
    }

    /**
     * @param string $datasource
     * @return string
     */
    public static function regexOperator(string $datasource = 'default'): string
    {
        try {
            return self::driverName($datasource) === 'pgsql' ? '~' : 'REGEXP';
        } catch (ReflectionException) {
        }
        return 'REGEXP';
    }

    /**
     * @param string $datasource
     * @return PDO
     * @throws ReflectionException
     */
    public static function getPdo(string $datasource = 'default'): PDO
    {
        $driver = self::getDriver($datasource);
        if (!$driver->isConnected()) {
            $driver->connect();
        }
        $reflection = new ReflectionClass($driver);
        return $reflection->getMethod('getPdo')->invokeArgs($driver, []);
    }

    /**
     * @param string $string
     * @param string $type
     * Provides a data type hint for drivers that have alternate quoting styles.
     * @param string $datasource
     * @return string
     * @throws ReflectionException
     */
    public static function quote(
        string $string,
        string $type = PDO::PARAM_STR,
        string $datasource = 'default'
    ): string {
        return self::getPdo($datasource)->quote($string, $type);
    }
}
