<?php

/**
 * AsalaeCore\View\Helper\TranslateHelper
 */

namespace AsalaeCore\View\Helper;

use Cake\I18n\I18n;
use Cake\View\Helper;

/**
 * Centralise certaines traductions
 *
 * @category Helper
 *
 * @author    Libriciel SCOP <contact@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   https://www.gnu.org/licenses/agpl-3.0.txt
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class TranslateHelper extends Helper
{
    /**
     * Helpers utilisés
     *
     * @var array
     */
    public array $helpers = ['Html'];

    /**
     * Permet d'obtenir la traduction pour un controller / action dans un
     * contexte de droits
     *
     * @param string      $controller
     * @param string|null $action
     * @return string
     */
    public function permission(string $controller, string $action = null): string
    {
        if ($action === null) {
            if ($controller == 'Agreements') {
                return __d('permission', "Accords de versement");
            }
            return I18n::getTranslator('permission')->translate($controller);
        }
        if ($controller === 'Download') {
            switch ($action) {
                case 'file':
                    return __dx('permission', 'Download', 'file');
                case 'open':
                    return __dx('permission', 'Download', 'open');
                case 'thumbnailImage':
                    return __dx('permission', 'Download', 'thumbnailImage');
                case 'zip':
                    return __dx('permission', 'Download', 'zip');
            }
        }

        // Actions générique
        return match ($action) {
            'add', 'add1' => __d('permission', "Ajouter"),
            'delete' => __d('permission', "Supprimer"),
            'edit' => __d('permission', "Modifier"),
            'index' => __d('permission', "Lister"),
            'view' => __d('permission', "Visualiser"),
            'api.create' => __d('permission', "Création via Web service"),
            'api.read' => __d('permission', "Lecture via Web service"),
            'api.update' => __d('permission', "Mise à jour via Web service"),
            'api.delete' => __d('permission', "Suppression via Web service"),
            default => I18n::getTranslator('permission')->translate($action, ['_context' => $controller]),
        };
    }

    /**
     * Traduction des options de permission
     * @param string $name
     * @return string
     */
    public function optionsPerms(string $name): string
    {
        return match ($name) {
            'versant' => __("Versant"),
            'producteur' => __("Producteur"),
            'admin' => __("Administrateur"),
            default => I18n::getTranslator()->translate($name),
        };
    }
}
