<?php

namespace AsalaeCore\Test\TestCase\Command;

use AsalaeCore\Command\FixturesDatabaseCommand;
use AsalaeCore\ORM\TableRegistry;
use AsalaeCore\TestSuite\TestCase;
use Cake\Console\TestSuite\ConsoleIntegrationTestTrait;
use Cake\Core\Configure;
use Libriciel\Filesystem\Utility\Filesystem;

class FixturesDatabaseCommandTest extends TestCase
{
    use ConsoleIntegrationTestTrait;

    public array $fixtures = [
        'app.Acos',
        'app.Agreements',
        'app.ArchivingSystems',
        'app.ArosAcos',
        'app.Aros',
        'app.AuthSubUrls',
        'app.AuthUrls',
        'app.Configurations',
        'app.Counters',
        'app.CronExecutions',
        'app.Crons',
        'app.Eaccpfs',
        'app.FileExtensions',
        'app.FileExtensionsPronoms',
        'app.Fileuploads',
        'app.Filters',
        'app.KeywordLists',
        'app.Keywords',
        'app.Ldaps',
        'app.Mediainfos',
        'app.MediainfoTexts',
        'app.MediainfoVideos',
        'app.Notifications',
        'app.OrgEntities',
        'app.OutgoingTransfers',
        'app.Profiles',
        'app.Pronoms',
        'app.Roles',
        'app.SavedFilters',
        'app.Sequences',
        'app.ServiceLevels',
        'app.Sessions',
        'app.Siegfrieds',
        'app.Timestampers',
        'app.TransferAttachments',
        'app.Transfers',
        'app.TypeEntities',
        'app.Users',
        'app.Versions',
        'app.Webservices',
    ];

    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
        if (is_dir(TMP_TESTDIR)) {
            Filesystem::remove(TMP_TESTDIR);
        }
        mkdir(TMP_TESTDIR);
        $controllerJson = TMP_TESTDIR . DS . 'testunit-controllers.json';
        Configure::write('App.paths.controllers_rules', $controllerJson);
        file_put_contents(
            $controllerJson,
            json_encode(
                [
                    'MyPlugin.Sample' => [
                        'test' => [
                            'invisible' => '0',
                            'accesParDefaut' => '0',
                            'commeDroits' => 'MyPlugin.Sample::add',
                        ],
                        'add' => [
                            'invisible' => '0',
                            'accesParDefaut' => '1',
                            'commeDroits' => '',
                        ],
                    ],
                ]
            )
        );
        $apiJson = TMP_TESTDIR . DS . 'testunit-api.json';
        Configure::write('App.paths.apis_rules', $apiJson);
        file_put_contents(
            $apiJson,
            json_encode(
                [
                    'MyPlugin.Webservices' => [
                        'default' => [
                            'invisible' => '0',
                            'accesParDefaut' => '1',
                            'commeDroits' => '',
                        ],
                        'test' => [
                            'invisible' => '0',
                            'accesParDefaut' => '0',
                            'commeDroits' => 'MyPlugin.Webservices::default',
                        ],
                    ],
                ]
            )
        );
    }

    /**
     * tearDown
     */
    public function tearDown(): void
    {
        parent::tearDown();
        $loc = $this->getTableLocator();
        foreach ($this->fixtures as $fixture) {
            [, $modelName] = explode('.', $fixture);
            $loc->remove($modelName);
        }
        $loc->clear();
        if (is_dir(TMP_TESTDIR)) {
            Filesystem::remove(TMP_TESTDIR);
        }
    }

    /**
     * @return void
     * @see FixturesDatabaseCommand
     */
    public function testexecute(): void
    {
        $ArosAcos = $this->fetchTable('ArosAcos');
        $initialCount = $ArosAcos->find()->count();
        $exportJsonPath = TMP_TESTDIR . DS . 'testunit-export.json';
        $exportJson = escapeshellarg($exportJsonPath);
        $this->exec('roles_perms export ' . $exportJson);
        $this->getTableLocator()->remove('Acos');
        $this->exec('fixtures database --datasource test --with-volumes --export-role-json ' . $exportJson);
        $this->assertOutputContains($exportJsonPath);
        $this->assertOutputContains(__("L'import s'est effectué avec succès"));
        $this->assertGreaterThan($initialCount, $ArosAcos->find()->count());
    }
}
