<?php

namespace AsalaeCore\Test\TestCase\Command;

use AsalaeCore\Model\Table\OrgEntitiesTable;
use AsalaeCore\TestSuite\TestCase;
use Cake\Console\TestSuite\ConsoleIntegrationTestTrait;
use Cake\Core\Configure;
use Cake\ORM\TableRegistry;
use Composer\Autoload\ClassLoader;

class TreeCommandTest extends TestCase
{
    use ConsoleIntegrationTestTrait;

    public array $fixtures = [
        'app.Aros',
        'app.Keywords',
        'app.OrgEntities',
        'app.Roles',
    ];

    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
        /** @var ClassLoader $loader */
        $loader = include ROOT . DS . 'vendor' . DS . 'autoload.php';
        Configure::write(
            'Tree.model_path',
            dirname(realpath($loader->findFile(OrgEntitiesTable::class)))
        );
    }

    /**
     * tearDown
     */
    public function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * test Repair
     */
    public function testRepair(): void
    {
        $OrgEntities = TableRegistry::getTableLocator()->get('OrgEntities');
        $OrgEntities->updateAll(['lft' => null, 'rght' => null], []);
        $this->assertCount($OrgEntities->find()->count(), $OrgEntities->find()->where(['lft IS' => null]));
        $this->exec('tree repair org_entities');
        $this->assertOutputContains('OrgEntities');
        $this->assertCount(0, $OrgEntities->find()->where(['lft IS' => null]));
    }

    /**
     * test RepairAll
     */
    public function testRepairAll(): void
    {
        $OrgEntities = TableRegistry::getTableLocator()->get('OrgEntities');
        $OrgEntities->updateAll(['lft' => null, 'rght' => null], []);
        $this->assertCount($OrgEntities->find()->count(), $OrgEntities->find()->where(['lft IS' => null]));
        $this->exec('tree repair all');
        $this->assertOutputContains('OrgEntities');
        $this->assertCount(0, $OrgEntities->find()->where(['lft IS' => null]));
    }
}
