<?php

namespace AsalaeCore\Test\TestCase\Controller\Component;

use AsalaeCore\Controller\Component\AjaxPaginatorComponent;
use AsalaeCore\TestSuite\TestCase;
use Cake\ORM\TableRegistry;

class AjaxPaginatorComponentTest extends TestCase
{
    use ComponentTestTrait;

    /**
     * @var AjaxPaginatorComponent $component
     */
    public $component = null;

    /**
     * @var string|AjaxPaginatorComponent
     */
    public $componentClassname = AjaxPaginatorComponent::class;

    public array $fixtures = [
        'app.Crons',
        'app.CronExecutions',
    ];

    /**
     * test Paginate
     */
    public function testPaginate(): void
    {
        $this->setComponent(
            [
                'query' => ['sort' => 'name', 'direction' => 'desc'],
            ]
        );
        $model = TableRegistry::getTableLocator()->get('Crons');
        $query = $model->selectQuery();
        $results = $this->component->paginate($query);
        $this->assertGreaterThanOrEqual(1, $results->count());

        $this->setComponent(
            [
                'query' => ['sort' => 'CronExecutions.state', 'direction' => 'asc'],
            ]
        );
        $query = $model->selectQuery()->innerJoinWith('CronExecutions');
        $results = $this->component->paginate($query);
        $this->assertGreaterThanOrEqual(1, $results->count());
    }

    /**
     * test Json
     */
    public function testJson(): void
    {
        $this->setComponent(
            [
                'query' => ['sort' => 'name', 'direction' => 'desc'],
                'request_headers' => [
                    'X-Paginator-Count' => 'true',
                ],
            ]
        );
        $model = TableRegistry::getTableLocator()->get('Crons');
        $query = $model->selectQuery();
        $results = $this->component->json($query);
        $json = json_decode($results->getBody()->getContents());
        $this->assertGreaterThanOrEqual(1, count($json));
    }
}
