<?php

namespace AsalaeCore\Test\TestCase\Controller\Component;

use AsalaeCore\Controller\Component\JstableComponent;
use AsalaeCore\TestSuite\TestCase;

class JstableComponentTest extends TestCase
{
    use ComponentTestTrait;

    /**
     * @var JstableComponent $component
     */
    public $component = null;

    /**
     * @var string|JstableComponent
     */
    public $componentClassname = JstableComponent::class;

    public array $fixtures = [
        'app.Aros',
        'app.Users',
        'app.Roles',
    ];

    /**
     * test ConfigurablePagination
     */
    public function testConfigurablePagination(): void
    {
        $session = [
            'Auth' => [
                'cookies' => [
                    'table-generator-config-table-test-max-per-page' => 'foo',
                ],
            ],
        ];
        $this->setComponent(['session' => $session]);
        $this->component->configurablePagination('test');
        $this->assertEquals([], $this->controller->paginate);

        $session = [
            'Auth' => [
                'cookies' => [
                    'table-generator-config-table-test-max-per-page' => '100',
                ],
            ],
        ];
        $this->setComponent(['session' => $session]);
        $this->component->configurablePagination('test');
        $this->assertEquals(['limit' => '100'], $this->controller->paginate);

        $cookies = [
            'table-generator-config-table-test-max-per-page' => '123',
        ];
        $this->setComponent(['cookies' => $cookies]);
        $this->component->configurablePagination('test');
        $this->assertEquals(['limit' => '123'], $this->controller->paginate);
    }

    /**
     * test AjaxPagination
     */
    public function testAjaxPagination(): void
    {
        $this->setComponent(['environment' => ['HTTP_X_REQUESTED_WITH' => 'XMLHttpRequest']]);
        $this->component->ajaxPagination();
        $this->assertEquals('ajax_test', $this->controller->viewBuilder()->getTemplate());
    }

    public function testsaveCookies(): void
    {
        $Users = $this->fetchTable('Users');
        $Users->updateAll(['cookies' => '[]'], ['id' => 1]);
        $options = [
            'cookies' => [
                'table-generator-config-table-test-max-per-page' => 'foo',
            ],
            'session' => [
                'Auth' => [
                    'id' => 1,
                ],
            ],
        ];
        $this->setComponent($options);
        $this->component->saveCookies();
        $user = $Users->get(1);
        $this->assertStringContainsString('foo', $user->get('cookies'));
    }
}
