<?php

namespace AsalaeCore\Test\TestCase\Controller\Component;

use AsalaeCore\Controller\Component\ModalComponent;
use AsalaeCore\TestSuite\TestCase;
use Cake\ORM\TableRegistry;

class ModalComponentTest extends TestCase
{
    use ComponentTestTrait;

    /**
     * @var ModalComponent $component
     */
    public $component = null;

    /**
     * @var string|ModalComponent
     */
    public $componentClassname = ModalComponent::class;

    public array $fixtures = [
        'app.Crons',
        'app.CronExecutions',
    ];

    /**
     * test Success
     */
    public function testSuccess(): void
    {
        $this->component()->success();
        $this->assertEquals('true', $this->controller->getResponse()->getHeaderLine('X-Asalae-Success'));

        $this->component()->success(false);
        $this->assertEquals('false', $this->controller->getResponse()->getHeaderLine('X-Asalae-Success'));
    }

    /**
     * component
     */
    private function component(): ModalComponent
    {
        /** @var ModalComponent $component */
        $component = $this->setComponent();
        return $component;
    }

    /**
     * test Fail
     */
    public function testFail(): void
    {
        $this->component()->fail();
        $this->assertEquals('false', $this->controller->getResponse()->getHeaderLine('X-Asalae-Success'));
    }

    /**
     * test Save
     */
    public function testSave(): void
    {
        $Crons = TableRegistry::getTableLocator()->get('Crons');
        $cron = $Crons->get(1);
        $callbacked = false;
        $this->component()->save(
            $Crons,
            $cron,
            ['foo' => 'bar'],
            ['checkExisting' => false],
            function ($v) use (&$callbacked) {
                $callbacked = true;
                return $v;
            }
        );
        $this->assertTrue($callbacked);
        $this->assertEquals('true', $this->controller->getResponse()->getHeaderLine('X-Asalae-Success'));
        $this->assertTrue($this->component->lastSaveIsSuccess());

        $cron->set('biz');
        $cron->setError('biz', 'bar');
        $callbacked = false;
        $this->component()->save(
            $Crons,
            $cron,
            ['foo' => 'bar'],
            ['checkExisting' => false],
            function ($v) use (&$callbacked) {
                $callbacked = true;
                return $v;
            }
        );
        $this->assertFalse($callbacked);
        $this->assertEquals('false', $this->controller->getResponse()->getHeaderLine('X-Asalae-Success'));
        $this->assertFalse($this->component->lastSaveIsSuccess());
    }

    /**
     * test Step
     */
    public function testStep(): void
    {
        $this->component()->step('foo("bar")');
        $this->assertEquals('foo("bar")', $this->controller->getResponse()->getHeaderLine('X-Asalae-Step'));
    }
}
