<?php

namespace AsalaeCore\Test\TestCase\Controller\Component;

use AsalaeCore\Controller\Component\Reader\ImageReader;
use AsalaeCore\Controller\Component\Reader\OpenDocumentReader;
use AsalaeCore\Controller\Component\Reader\PdfReader;
use AsalaeCore\Controller\Component\Reader\ReaderInterface;
use AsalaeCore\Controller\Component\ReaderComponent;
use AsalaeCore\TestSuite\TestCase;
use Cake\Core\Configure;
use Cake\Http\Exception\NotImplementedException;

class ReaderComponentTest extends TestCase
{
    use ComponentTestTrait;

    /**
     * @var ReaderComponent $component
     */
    public $component = null;

    /**
     * @var string|ReaderComponent
     */
    public $componentClassname = ReaderComponent::class;

    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
        Configure::write(
            [
                'Readers' => [
                    'pdf' => PdfReader::class,
                    'open' => OpenDocumentReader::class,
                    'image' => ImageReader::class,
                ],
            ]
        );
    }

    /**
     * test GetReader
     */
    public function testGetReader(): void
    {
        $this->assertNotEmpty(ReaderComponent::getReaders());
    }

    /**
     * test CanRead
     */
    public function testCanRead(): void
    {
        $this->component();
        $this->assertTrue($this->component->canRead('application/pdf-a'));
        $this->assertFalse($this->component->canRead('application/foo'));
    }

    /**
     * component
     */
    private function component(): ReaderComponent
    {
        /** @var ReaderComponent $component */
        $component = $this->setComponent();
        return $component;
    }

    /**
     * test Get
     */
    public function testGet(): void
    {
        $this->assertInstanceOf(
            ReaderInterface::class,
            $this->component()->get('application/pdf-a')
        );
        $e = null;
        try {
            $this->component->get('application/foo');
        } catch (NotImplementedException $e) {
        }
        $this->assertInstanceOf(NotImplementedException::class, $e);
    }
}
