<?php

namespace AsalaeCore\Test\TestCase\Database\Type;

use AsalaeCore\Database\Type\DateTimeType;
use AsalaeCore\TestSuite\InvokePrivateTrait;
use AsalaeCore\TestSuite\TestCase;
use Cake\Core\Configure;
use Cake\Database\TypeFactory;
use Cake\I18n\DateTime as Time;
use DateTimeImmutable;
use DateTimeInterface;

class DateTimeTypeTest extends TestCase
{
    use InvokePrivateTrait;

    /**
     * test parseValue
     */
    public function testparseValue(): void
    {
        Configure::write('App.defaultLocale', 'fr_FR');

        TypeFactory::set('datetime', new DateTimeType('datetime'));
        /** @var DateTimeType $type */
        $type = TypeFactory::build('datetime');

        /** @var Time $result */
        $result = $this->invokeMethod($type, '_parseValue', ['1950-01-01']);
        $this->assertInstanceOf(DateTimeImmutable::class, $result);
        $this->assertStringContainsString(
            '1950-01-01T00:00:00',
            $result->format(DateTimeInterface::ATOM)
        );

        $result = $this->invokeMethod($type, '_parseValue', ['1950-01-01 11:50']);
        $this->assertInstanceOf(DateTimeImmutable::class, $result);
        $this->assertStringContainsString(
            '1950-01-01T11:50:00',
            $result->format(DateTimeInterface::ATOM)
        );

        $result = $this->invokeMethod($type, '_parseValue', ['1950-01-01T11:50:22+0200']);
        $this->assertInstanceOf(DateTimeImmutable::class, $result);
        $this->assertEquals('1950-01-01T11:50:22+02:00', $result->format(DateTimeInterface::ATOM));

        $result = $this->invokeMethod($type, '_parseValue', [new Time('08/07/2013 09:09')]);
        $this->assertInstanceOf(DateTimeInterface::class, $result);
        $this->assertStringContainsString(
            '2013-08-07T09:09:00',
            $result->format(DateTimeInterface::ATOM)
        );
    }
}
