<?php

namespace AsalaeCore\Test\TestCase\Datasource;

use AsalaeCore\TestSuite\TestCase;
use Cake\Datasource\Paging\Exception\PageOutOfBoundsException;
use Cake\Datasource\Paging\NumericPaginator;
use Cake\ORM\TableRegistry;

class PaginatorTest extends TestCase
{
    public array $fixtures = [
        'app.Users',
    ];

    /**
     * test Paginate
     */
    public function testPaginate(): void
    {
        $Users = TableRegistry::getTableLocator()->get('Users');
        $paginator = new NumericPaginator();
        $paginator->setConfig('allowedParameters', ['limit', 'sort', 'page', 'direction', 'sortable']);
        $results = $paginator->paginate($Users);
        $this->assertGreaterThanOrEqual(1, $results->count());

        $paginator->setConfig('order', ['name' => 'asc']);
        $query = $Users->find()->orderBy(['username']);
        $results = $paginator->paginate($query, ['sort' => 'id']);
        $this->assertGreaterThanOrEqual(1, $results->count());

        $results = $paginator->paginate($Users, ['sort' => 'foo', 'sortableFields' => ['foo' => 'Users.id']]);
        $this->assertGreaterThanOrEqual(1, $results->count());

        $this->expectException(PageOutOfBoundsException::class);
        $paginator->paginate($Users, ['page' => 1000, 'limit' => 1000]);
    }
}
