<?php

namespace AsalaeCore\Test\TestCase\Error;

use AsalaeCore\Error\AppExceptionRenderer;
use AsalaeCore\TestSuite\TestCase;
use Cake\Controller\ErrorController;
use Cake\Core\Configure;
use Cake\Http\Exception\ForbiddenException;
use Exception;
use ReflectionClass;

class AppExceptionRendererTest extends TestCase
{
    private array $templates;
    /**
     * @return void
     */
    public function setUp(): void
    {
        parent::setUp();
        Configure::write('debug', true); // assure le rendu complet
        Configure::write('Error.errorController', ErrorController::class);
        $this->templates = Configure::read('App.paths.templates');
    }

    /**
     * @return void
     */
    public function tearDown(): void
    {
        parent::tearDown();
        Configure::write('App.paths.templates', $this->templates);
        Configure::write('debug', true);
    }

    /**
     * @return void
     */
    public function testRender(): void
    {
        $renderer = new AppExceptionRenderer(new Exception('test', 403));
        $reflection = new ReflectionClass($renderer);
        $property = $reflection->getProperty('controller');
        /** @var ErrorController $controller */
        $controller = $property->getValue($renderer);
        // défini le layout sur tests/templates/layout/error.php pour avoir un affichage
        $controller->viewBuilder()->setLayout('testerror');
        $response = $renderer->render();
        $body = $response->getBody();
        $body->rewind();
        $content = $body->getContents();
        $this->assertStringContainsString('Testerror', $content);
    }

    /**
     * @return void
     */
    public function testRenderMissingTemplate(): void
    {
        Configure::write('App.paths.templates', [TESTS . 'templates/']);
        $exception = new ForbiddenException('test', 403);
        $reflection = new ReflectionClass($exception);
        $property = $reflection->getProperty('_messageTemplate');
        $property->setValue($exception, 'thistemplatedoesnotexist');
        $renderer = new AppExceptionRenderer($exception);
        Configure::write('debug', false); // evite le debug du missing template
        $response = $renderer->render();
        Configure::write('debug', true);
        $body = $response->getBody();
        $body->rewind();
        $content = $body->getContents();
        $this->assertStringContainsString('test Fatal error', $content);
    }
}
