<?php

namespace AsalaeCore\Test\TestCase\Factory;

use AsalaeCore\Factory;
use AsalaeCore\Test\Mock\TestPrivateUtility;
use AsalaeCore\TestSuite\TestCase;
use Exception;

class UtilityTest extends TestCase
{
    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
        Factory\Utility::addNamespace("AsalaeCore\Test\Mock");
    }

    /**
     * tearDown
     */
    public function tearDown(): void
    {
        parent::tearDown();
        Factory\Utility::reset();
    }

    /**
     * test Get
     */
    public function testGet(): void
    {
        $utility = Factory\Utility::get('TestUtility');
        $this->assertInstanceOf('AsalaeCore\Test\Mock\TestUtility', $utility);
        $this->assertTrue($utility->main());

        $utility = Factory\Utility::get('TestPrivateUtility');
        /** @var TestPrivateUtility $instance */
        $instance = $utility->getInstance();
        $this->assertInstanceOf('AsalaeCore\Factory\UtilityObject', $utility);
        $this->assertTrue($instance->main());

        $utility = Factory\Utility::get('TestStaticUtility');
        $this->assertInstanceOf('AsalaeCore\Factory\UtilityObject', $utility);
        $this->assertTrue($utility->main());
    }

    /**
     * test GetException
     */
    public function testGetException(): void
    {
        $this->expectException(Exception::class);
        Factory\Utility::get('NotExist');
    }

    /**
     * test Set
     */
    public function testSet(): void
    {
        $instance = 'AsalaeCore\Test\Mock\TestStaticUtility';
        Factory\Utility::set('TestUtility', $instance);
        $this->assertEquals($instance, Factory\Utility::get('TestUtility')->name());

        $instance = 'TestPrivateUtility';
        $set = Factory\Utility::set('TestUtility', $instance);
        $utility = Factory\Utility::get('TestUtility');

        /** @var TestPrivateUtility $instance */
        $instance = $utility->getInstance();
        $this->assertEquals('AsalaeCore\Test\Mock\TestPrivateUtility', $instance->name());
        $this->assertEquals($utility->getInstance(), $set->getInstance());
    }
}
