<?php

namespace AsalaeCore\Test\TestCase\Http;

use AsalaeCore\Http\ConnSession;
use AsalaeCore\TestSuite\TestCase;

class ConnSessionTest extends TestCase
{
    /**
     * test Timeout
     */
    public function testTimeout(): void
    {
        $defaultSession = [
            'cookie' => 'PHPSESSID',
            'ini' => [
                'session.use_trans_sid' => 0,
                'session.cookie_secure' => 1,
                'session.name' => 'PHPSESSID',
                'session.cookie_httponly' => 1,
            ],
            'defaults' => 'php',
            'cookiePath' => '/',
            'timeout' => (int)ini_get('session.gc_maxlifetime') / 60,
        ];
        $session = new ConnSession($defaultSession);
        $this->assertFalse($session->timedOut());

        $session->write('Config.time', (int)ini_get('session.gc_maxlifetime') + 600);
        $this->assertTrue($session->timedOut());
    }

    /**
     * test Timeleft
     */
    public function testTimeleft(): void
    {
        $defaultSession = [
            'cookie' => 'PHPSESSID',
            'ini' => [
                'session.use_trans_sid' => 0,
                'session.cookie_secure' => 1,
                'session.name' => 'PHPSESSID',
                'session.cookie_httponly' => 1,
            ],
            'defaults' => 'php',
            'cookiePath' => '/',
            'timeout' => (int)ini_get('session.gc_maxlifetime') / 60,
        ];
        $session = new ConnSession($defaultSession);
        $session->write('Config.time', time() - (int)ini_get('session.gc_maxlifetime') + 300);
        $this->assertWithinRange(300, $session->timeLeft(), 1);
    }
}
