<?php

namespace AsalaeCore\Test\TestCase\MinkSuite;

use AsalaeCore\MinkSuite\MinkCase;
use AsalaeCore\TestSuite\IntegrationCronTrait;
use AsalaeCore\TestSuite\InvokePrivateTrait;
use AsalaeCore\TestSuite\TestCase;
use Behat\Mink\Element\NodeElement;
use Behat\Mink\Mink;
use Behat\Mink\Selector\SelectorsHandler;
use Behat\Mink\Session;
use Cake\Console\ConsoleIo;
use Cake\Console\TestSuite\StubConsoleInput as ConsoleInput;
use Cake\Console\TestSuite\StubConsoleOutput as ConsoleOutput;
use Cake\Utility\Hash;
use Closure;
use DMore\ChromeDriver\ChromeDriver;
use PHPUnit\Framework\MockObject\Exception;
use PHPUnit\Framework\MockObject\MockObject;

class MinkCaseDoTraitTest extends TestCase
{
    use IntegrationCronTrait;
    use InvokePrivateTrait;

    public $minkCase;
    private $values = [];
    private $clicks = [];
    private $waits = [];
    private $scripts = [];
    private $mocks = [];

    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
    }

    /**
     * tearDown
     */
    public function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * test doLogin
     */
    public function testdoLogin(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'doLogin');
        $this->assertEquals(['admin', 'admin'], $this->values);
    }

    /**
     * Donne une instance mocké de mink
     * @param array $params
     * @return MinkCase
     * @throws Exception
     */
    private function getMink(array $params = []): Minkcase
    {
        $out = new ConsoleOutput();
        $err = new ConsoleOutput();
        $io = new ConsoleIo($out, $err, new ConsoleInput([]));

        $driverMink = $this->createMock(ChromeDriver::class);
        $sessionMink = $this->createMock(Session::class);
        $mink = $this->createMock(Mink::class);

        $this->values = [];
        $this->clicks = [];
        $this->waits = [];
        $this->scripts = [];

        $default = [
            'driverMink' => [
                'getCurrentUrl' => 'users/login',
                'find' => [$this->getNode('my_input')],
            ],
            'sessionMink' => [
                'getDriver' => $driverMink,
                'getSelectorsHandler' => new SelectorsHandler(),
                'wait' => fn($v, $cond) => $this->waits[] = [$v, $cond],
                'executeScript' => fn($v) => $this->scripts[] = $v,
            ],
            'mink' => [
                'getSession' => $sessionMink,
            ],
        ];
        $params = Hash::merge($default, $params);

        $mocks = [
            'driverMink' => $driverMink,
            'sessionMink' => $sessionMink,
            'mink' => $mink,
        ];
        /** @var MockObject $mock */
        foreach ($mocks as $name => $mock) {
            foreach ($params[$name] as $fn => $value) {
                if ($value instanceof Closure) {
                    $mock->method($fn)->willReturnCallback($value);
                } elseif ($value === $mock) {
                    $mock->method($fn)->willReturnSelf();
                } else {
                    $mock->method($fn)->willReturn($value);
                }
            }
            $this->mocks[$name] = $mock;
        }
        return new MinkCase('test', $mink, [], $io);
    }

    /**
     * getNode
     * @param string $text
     * @param string $value
     * @param array  $params
     * @return NodeElement
     * @throws Exception
     */
    private function getNode(string $text, string $value = 'value', array $params = []): NodeElement
    {
        $node = $this->createMock(NodeElement::class);
        $default = [
            'setValue' => fn($v) => $this->values[] = $v,
            'click' => fn() => $this->clicks[] = true,
            'getParent' => $node,
            'findAll' => [$node],
            'isVisible' => true,
            'getText' => $text,
            'getAttribute' => 'id',
            'getValue' => $value,
            'hasAttribute' => true,
            'hasClass' => true,
        ];
        $params = Hash::merge($default, $params);
        foreach ($params as $fn => $value) {
            if ($value instanceof Closure) {
                $node->method($fn)->willReturnCallback($value);
            } elseif ($value === $node) {
                $node->method($fn)->willReturnSelf();
            } else {
                $node->method($fn)->willReturn($value);
            }
        }
        return $node;
    }

    /**
     * test doClick
     */
    public function testdoClick(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'doClick', ['.foo']);
        $this->assertCount(1, $this->clicks);
    }

    /**
     * test waitModalOpen
     */
    public function testwaitModalOpen(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'waitModalOpen', [500]);
        $this->assertEquals(
            [
                [500, 'minkModalOpenned()'],
                [400, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test waitUploadComplete
     */
    public function testwaitUploadComplete(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'waitUploadComplete', [500]);
        $this->assertEquals(
            [
                [10, 'false'],
                [500, 'minkFileUploading === false'],
                [400, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test waitAjaxComplete
     */
    public function testwaitAjaxComplete(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'waitAjaxComplete', [500]);
        $this->assertEquals(
            [
                [500, 'minkAjaxRunning === false && customAjaxRunning === false'],
                [500, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test waitModalClose
     */
    public function testwaitModalClose(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'waitModalClose', [500]);
        $this->assertEquals(
            [
                [500, "$('.modal:visible').length === 0"],
            ],
            $this->waits
        );
    }

    /**
     * test setFormFieldValue
     */
    public function testsetFormFieldValue(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setFormFieldValue', ['my_input', 'value']);
        $this->assertEquals(['value'], $this->values);
    }

    /**
     * test setNamelessFormFieldValue
     */
    public function testsetNamelessFormFieldValue(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setNamelessFormFieldValue', ['my_input', 'value']);
        $this->assertEquals(['value'], $this->values);
    }

    /**
     * test setFormSelectValueByText
     */
    public function testsetFormSelectValueByText(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setFormSelectValueByText', ['my_input', 'my_input']);
        $this->assertEquals(['value'], $this->values);
    }

    /**
     * test setFormFieldValues
     */
    public function testsetFormFieldValues(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setFormFieldValues', ['my_input', ['value1', 'value2']]);
        $this->assertCount(2, $this->scripts);
    }

    /**
     * test setFormFieldValueById
     */
    public function testsetFormFieldValueById(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setFormFieldValueById', ['my_input', 'value']);
        $this->assertEquals(['value'], $this->values);
    }

    /**
     * test setInputValue
     */
    public function testsetInputValue(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'setInputValue', ['css', '.my_input', 'value']);
        $this->assertEquals(['value'], $this->values);
    }

    /**
     * test requiredIdName
     */
    public function testrequiredIdName(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'requiredIdName', ['.my_input']);
        $this->assertCount(1, $this->scripts);
        $this->assertEquals(
            [
                [10, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test removeRequiredIdName
     */
    public function testremoveRequiredIdName(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'removeRequiredIdName');
        $this->assertCount(1, $this->scripts);
    }

    /**
     * test submitModal
     */
    public function testsubmitModal(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'submitModal', [500]);
        $this->assertCount(1, $this->clicks);
        $this->assertEquals(
            [
                [500, 'minkAjaxRunning === false && customAjaxRunning === false'],
                [500, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test closeModal
     */
    public function testcloseModal(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'closeModal', [500]);
        $this->assertCount(1, $this->clicks);
        $this->assertEquals(
            [
                [500, 'minkAjaxRunning === false && customAjaxRunning === false'],
                [500, 'false'],
            ],
            $this->waits
        );
    }

    /**
     * test doClickUserMenu
     */
    public function testdoClickUserMenu(): void
    {
        $minkCase = $this->getMink();
        $this->invokeMethod($minkCase, 'doClickUserMenu', ['my_input']);
        $this->assertCount(2, $this->clicks);
    }

    /**
     * test doClickMenu
     */
    public function testdoClickMenu(): void
    {
        $params = [
            'driverMink' => [
                'find' => [
                    $this->getNode(
                        'title',
                        'value',
                        [
                            'findAll' => [$this->getNode('my_input')],
                            'getParent' => $this->getNode('my_input'),
                        ]
                    ),
                ],
            ],
        ];
        $minkCase = $this->getMink($params);
        $this->invokeMethod($minkCase, 'doClickMenu', ['title', 'my_input']);
        $this->assertCount(2, $this->clicks);
    }
}
