<?php

namespace AsalaeCore\Test\TestCase\View\Helper;

use AsalaeCore\Http\AppServerRequest;
use AsalaeCore\TestSuite\TestCase;
use AsalaeCore\View\Helper\FilterHelper;
use Cake\Core\Configure;
use Cake\Datasource\EntityInterface;
use Cake\View\Helper\UrlHelper;
use Cake\View\View;
use Exception;
use PHPUnit\Framework\MockObject\Exception as MockException;
use ReflectionClass;

class FilterHelperTest extends TestCase
{
    public array $fixtures = [
        'app.Filters',
        'app.SavedFilters',
    ];

    /**
     * setup
     */
    public function setUp(): void
    {
        parent::setUp();
        Configure::write('App.defaultLocale', 'fr_FR');
    }

    /**
     * test GenerateLoadedFilterContent
     */
    public function testGenerateLoadedFilterContent(): void
    {
        $Filter = $this->getFilterHelper()
            ->create('test')
            ->filter('test', ['label' => 'foo']);

        // Pas de filtre
        $result = $Filter->generateLoadedFilterContent();
        $this->assertEmpty($result);

        // Filtre simple
        $Filter = $this->getFilterHelper(['test' => 'bar'])
            ->create('test')
            ->filter('test', ['label' => 'foo']);
        $result = $Filter->generateLoadedFilterContent();
        $this->assertStringContainsString(
            '<input type="text" name="test[0]"',
            $result
        );

        $Filter = $this->getFilterHelper(
            ['created' => '01/01/2020', 'dateoperator_created' => '<=']
        )
            ->create('test')
            ->filter(
                'created',
                ['label' => 'created', 'type' => 'date']
            );
        $result = $Filter->generateLoadedFilterContent();
        $this->assertStringContainsString(
            '01/01/2020',
            $result
        );
        $this->assertStringContainsString(
            '<input type="text" name="created[0]"',
            $result
        );
        $this->assertStringContainsString(
            '<select name="dateoperator_created[0]"',
            $result
        );
    }

    /**
     * getFilterHelper
     * @param array $queryParams
     * @return FilterHelper
     */
    private function getFilterHelper(array $queryParams = []): FilterHelper
    {
        $FilterHelper = new FilterHelper(
            new View(
                (new AppServerRequest())->withQueryParams($queryParams)
            )
        );
        try {
            $reflection = new ReflectionClass($FilterHelper);
            $property = $reflection->getProperty('helperInstances');
            $helperInstances = $property->getValue($FilterHelper);
            $helperInstances['Url'] = $this->createMock(UrlHelper::class);
            $helperInstances['Url']->method('build')->willReturn('/foo/bar');
            $property->setValue($FilterHelper, $helperInstances);
        } catch (MockException) {
        }
        return $FilterHelper;
    }

    /**
     * @return void
     */
    public function testSaves(): void
    {
        $SavedFilters = $this->fetchTable('SavedFilters');
        /** @var EntityInterface[] $saves */
        $saves = $SavedFilters->find()
            ->where(
                [
                    'user_id' => 1,
                    'controller' => 'ctrl',
                    'action' => 'act',
                ]
            )
            ->contain(['Filters'])
            ->all()
            ->toArray();

        $Filter = $this->getFilterHelper()->create('test');
        $Filter->saves($saves);
        $this->assertNotEmpty($Filter->saveList);

        $Filter->loadFilters($saves[0]);
        $this->assertEquals($saves[0]->id, $Filter->selectedSave);
        $this->assertNotEmpty($Filter->loadedFilter);
    }

    /**
     * @return void
     */
    public function testToString(): void
    {
        $FilterHelper = $this->getFilterHelper();
        $Filter = $FilterHelper->create('test');
        $this->assertNotEmpty((string)$Filter);
    }

    /**
     * @return void
     * @throws Exception
     */
    public function testgenerateSection(): void
    {
        $FilterHelper = $this->getFilterHelper();
        $Filter = $FilterHelper->create('test');
        $Filter->permanentFilter('foo', ['label' => 'foo', 'wildcard']);
        $this->assertNotEmpty($Filter->generateSection());
    }
}
