<?php

namespace AsalaeCore\Test\TestCase\View\Helper;

use AsalaeCore\Controller\Component\AjaxPaginatorComponent;
use AsalaeCore\TestSuite\TestCase;
use AsalaeCore\View\Helper\PaginatorHelper;
use Cake\Controller\ComponentRegistry;
use Cake\Controller\Controller;
use Cake\Core\Configure;
use Cake\Http\ServerRequest;
use Cake\Routing\Router;
use PHPUnit\Framework\MockObject\Exception;
use Random\RandomException;

class PaginatorHelperTest extends TestCase
{
    /**
     * @return void
     * @throws Exception
     */
    public function setUp(): void
    {
        parent::setUp();
        Configure::write('App.defaultLocale', 'fr_FR');

        // note: @internal
        $collection = $this->createMock('\Cake\Routing\RouteCollection');
        $collection->method('parseRequest')->willReturn(
            [
                'pass' => [],
                'controller' => 'Home',
                'action' => 'index',
                'plugin' => null,
                '_matchedRoute' => '/',
                '_route' => $collection,
            ]
        );
        Router::setRouteCollection($collection);
    }

    /**
     * @return void
     * @throws RandomException
     */
    public function testblocPagination(): void
    {
        $results = [];
        for ($i = 1; $i <= 100; $i++) {
            $results[] = [
                'id' => $i,
                'name' => 'Name_' . bin2hex(random_bytes(4)),
            ];
        }
        $helper = $this->getPaginatorHelper($results, 999);
        $bloc = $helper->blocPagination();
        $message = __(
            'Affichage des résultats de {0} à {1} sur les<br>{2} résultats',
            '<i class="start">1</i>',
            '<i class="end">100</i>',
            '<i class="count">999</i>'
        );
        $this->assertStringContainsString($message, $bloc);
    }

    /**
     * @return void
     */
    public function testsort(): void
    {
        $helper = $this->getPaginatorHelper([], 0);
        $sort = $helper->sort('foo');
        $this->assertStringContainsString('Foo', $sort);
        $this->assertStringContainsString('sortable', $sort);

        $params = $helper->getConfig('params');
        $params['sort'] = 'foo';
        $params['direction'] = 'desc';
        $params['alias'] = 'Mytable';
        $helper->setConfig('params', $params);
        $sort = $helper->sort('foo', 'Foo', ['sort' => 'foo', 'direction' => 'desc']);
        $this->assertStringContainsString('desc', $sort);
    }

    /**
     * @param array $paginatedData
     * @param int   $queryCount
     * @return PaginatorHelper
     */
    private function getPaginatorHelper(array $paginatedData, int $queryCount): PaginatorHelper
    {
        $controller = new Controller(new ServerRequest());
        $Collection = new ComponentRegistry($controller);
        $component = new AjaxPaginatorComponent($Collection);
        $component->setViewPaginator($paginatedData, $queryCount);
        $view = $controller->viewBuilder()->build();
        return new PaginatorHelper($view);
    }
}
